from pygame import draw, Rect
from Gfx import gfx

class Hud:
	"""
	Handle the display of on screen information.
	"""
	def __init__(self, world, player):
		self.world = world
		self.player = player
	
	def Draw(self):
		"""
		Draw all hud elements
		"""
		if self.player.strength < 0:
			txt = gfx.hfont.render("Oops, you are dead. :/", 1, (150, 10, 10))
	    		textpos = txt.get_rect()
			scrnpos = gfx.screen.get_rect()
			textpos.centerx = scrnpos.centerx
			textpos.centery = scrnpos.centery - gfx.screen.get_height() * 0.2
			gfx.screen.blit(txt, textpos)

		self.nearstuff = self.player.nearstuff
		####
		#### Show the plating and strength
		####
		txt = gfx.hfont.render("Strength: %d     Plating: %d" % (self.player.strength, self.player.plating), 1, (0, 0, 0))
    		textpos = txt.get_rect()
		textpos.left = gfx.screen.get_width() * 0.025
		textpos.bottom = gfx.screen.get_height() * 0.975
		gfx.screen.blit(txt, textpos)

		### Display the position of the player
		txt = gfx.cfont.render("you are here -> (%d, %d)" % (self.player.position[0], self.player.position[1]), 1, (150, 150, 150));
		textpos = txt.get_rect()
		textpos.right = gfx.screen.get_width() * 0.975
		textpos.bottom = gfx.screen.get_height() * 0.975
		gfx.screen.blit(txt, textpos)

		# if the player is standing next to something then show it
		####
		#### Draw nearstuff
		####
		if len(self.nearstuff):
			disparea = [gfx.screen.get_width() * 0.05, gfx.screen.get_height() * 0.7, gfx.screen.get_width() * 0.1 * len(self.nearstuff), gfx.screen.get_height() * 0.2]
			# draw a square near the bottom left of the screen showing the thing.
			gfx.screen.fill([255, 255, 255], disparea)
			draw.rect(gfx.screen, [0, 0, 0], disparea, 1)
			# draw the thing
			for n in self.nearstuff:
				# where on the screen to position it
				box = [gfx.screen.get_width() * (0.1 * (self.nearstuff.index(n) + 1)), gfx.screen.get_height() * 0.85]
				self.DrawItem(n, box)
			
		if self.player.actiontime:
			if type(self.player.actiontext) == list:
				# we have a poem
				for t in range(0, 3):
					txt = gfx.hfont.render(self.player.actiontext[t], 1, (0, 0, 0))
		 	   		textpos = txt.get_rect()
					scrnpos = gfx.screen.get_rect()
					textpos.centerx = scrnpos.centerx
					textpos.centery = scrnpos.centery + t * gfx.hfont.get_linesize()
					gfx.screen.blit(txt, textpos)
			else:
				# write the actionstext
				txt = gfx.hfont.render(self.player.actiontext, 1, (0, 0, 0))
		    		textpos = txt.get_rect()
				scrnpos = gfx.screen.get_rect()
				textpos.centerx = scrnpos.centerx
				textpos.centery = scrnpos.centery
				gfx.screen.blit(txt, textpos)
		
		####
		####	Draw inventory
		####
		# if the player is viewing their inventory then we need to draw it
		if self.player.viewingInventory:
			disparea = Rect(gfx.screen.get_width() * 0.55, gfx.screen.get_height() * 0.05, gfx.screen.get_width() * 0.4, gfx.screen.get_height() * 0.1 * (len(self.player.inventory) + 1))
			gfx.screen.fill([255, 255, 255], disparea)
			draw.rect(gfx.screen, [0, 0, 0], disparea, 1)
			
			# write the word 'inventory'
			txt = gfx.hfont.render("Inventory", 1, (0, 0, 0))
	    		textpos = txt.get_rect()
			textpos.centerx = disparea.centerx
			textpos.centery = gfx.screen.get_height() * 0.025
			gfx.screen.blit(txt, textpos)
			
			# now draw each object into the inventory and write it's name next to it
			for n in self.player.inventory:
				box = [gfx.screen.get_width() * 0.6, gfx.screen.get_height() * 0.2 + self.player.inventory.index(n) * 0.1 * gfx.screen.get_height()]
				if self.player.selected == self.player.inventory.index(n):
					draw.rect(gfx.screen, [255, 230, 230], Rect(box[0], box[1] - 0.1 * gfx.screen.get_height(), 0.34 * gfx.screen.get_width(), 0.1 * gfx.screen.get_height()), 0)
				self.DrawItem(n, box)
				
				if (n.istext, n.amount) in self.player.known:
					txt = gfx.font.render(n.description + " " + n.istext + (n.amount and (" (" + ((n.amount > 0) and "+" or "") + str(n.amount) + ")") or ""), 1, (0, 0, 0))
				else:
					txt = gfx.font.render(n.description, 1, (0, 0, 0))
		    		textpos = txt.get_rect()
				textpos.left = disparea.left + 0.1 * gfx.screen.get_width()
				textpos.centery = gfx.screen.get_height() * 0.1 * (self.player.inventory.index(n) + 1.5)
				gfx.screen.blit(txt, textpos)


	def DrawItem(self, item, box):
		"""
		Draws a single item in inventory view.
		item is the item to draw.
		Box is it's size.
		"""
		# box = [gfx.screen.get_width() * (0.1 * (self.nearstuff.index(n) + 1)), gfx.screen.get_height() * 0.9]
		for p in item.animations[item.animation][int(item.frame)]:
			points = [[item.scale * x * 3000 + box[0], item.scale * y * 3000 + box[1]] for x,y in p['points']]
			if len(points) > 1:
				if p['filled'] and len(points) > 2:
					draw.polygon(gfx.screen, p['color'], points, 0)
				draw.lines(gfx.screen, p['outline'], p['closed'], points, p['width'])

