from random import *
from math import *

"""
These are functions to make random data.
"""

# the namespace that this module is currently working in.
namespace = "default"

def SetNamespace(ns="default"):
	namespace = ns
	seed(namespace)

def SetPosition(x, y):
	"""
	Use the current position of an object to seed the random number generator.
	"""
	seed(x + y * 57)

def NameSpacedNoise(x, y):
	"""
	Always produces the same 'random' result depending on a two dimensional position
	and a name space.
	"""
	global namespace
	seed(namespace)
	seed(randint(0, 60000) * (x + y * 57))

def CosInterpolation(a, b, x):
	"""
	Interpolates between two points using a cosine.
	http://freespace.virgin.net/hugo.elias/models/m_perlin.htm
	"""
	ft = x * 3.1415927
	f = (1 - cos(ft)) * 0.5
	return  a*(1-f) + b*f

def Noise2D(x, y, ns=False):
	"""
	Two dimensional perlin noise. FOR THE WIN!
	http://freespace.virgin.net/hugo.elias/models/m_perlin.htm
	"""
	if ns:
		global namespace
		namespace = ns
	NameSpacedNoise(x, y)
	return random()
	#n = x + y * 57
	#n = (n<<13) ^ n;
	#return abs(( 1.0 - ( (n * (n * n * 15731 + 789221) + 1376312589) & 0x7fffffff) / 1073741824.0));

def SmoothNoise2D(x, y):
	corners = ( Noise2D(x-1, y-1)+Noise2D(x+1, y-1)+Noise2D(x-1, y+1)+Noise2D(x+1, y+1) ) / 16
	sides   = ( Noise2D(x-1, y)  +Noise2D(x+1, y)  +Noise2D(x, y-1)  +Noise2D(x, y+1) ) /  8
	center  =  Noise2D(x, y) / 4
	return corners + sides + center

def InterpolatedNoise2D(x, y, ns=False):
	if ns:
		global namespace
		namespace = ns
	
	intX = int(x)
	fracX = x - intX
	intY = int(y)
	fracY = y - intY
	
	v1 = SmoothNoise2D(intX, intY)
	v2 = SmoothNoise2D(intX + 1, intY)
	v3 = SmoothNoise2D(intX, intY + 1)
	v4 = SmoothNoise2D(intX + 1, intY + 1)
	
	i1 = CosInterpolation(v1, v2, fracX)
	i2 = CosInterpolation(v3, v4, fracX)
	
	return CosInterpolation(i1 ,i2 ,fracY)

def ScaledNoise2D(x, y, ns):
	return max(min(1.0, (InterpolatedNoise2D(x, y, ns) - 0.3) * 2.5), 0.0)
