from engine.Gfx import gfx
from engine.EventHandler import events, EventMonitor
from engine.Entity import Entity
from pygame.locals import *
from resources.gfx import echo, echowalk, echowalkup, echowalkdown, echohit, echodie
from Character import Character

class Robot(EventMonitor, Entity, Character):
	def __init__(self, world):
		EventMonitor.__init__(self)
		Entity.__init__(self, world)
		
		self.AddAnimation("stand", echo.pufferdata, [self.Normalise])
		self.AddAnimation("walkright", echowalk.pufferdata, [self.Normalise])
		self.AddAnimation("walkleft", echowalk.pufferdata, [self.Normalise, self.Reverse])
		self.AddAnimation("walkup", echowalkup.pufferdata, [self.Normalise])
		self.AddAnimation("walkdown", echowalkdown.pufferdata, [self.Normalise])
		self.AddAnimation("hitr", echohit.pufferdata, [self.Normalise])
		self.AddAnimation("hitl", echohit.pufferdata, [self.Normalise, self.Reverse])
		self.AddAnimation("dead", echodie.pufferdata, [self.Normalise])
		self.Animate("stand")
		
		self.velocity = 0.01
		self.scale = 0.2
		self.position = [0, 0]
		self.framerate = 0.6
		
		self.known = {}
		
		self.selected = None
		
		######
		# all the properties that a robot can have
		######
		# Item Entities that are near me on the ground
		self.nearstuff = []
		# A list of things that I have picked up
		self.inventory = []
		# how strong i am in fighting
		self.strength = 3
		# how long until i am no longer invulnerable
		self.invulnerable = 0
		# how long until i am no longer invisible
		self.invisible = 0
		# how long until i am no longer confused
		self.confused = 0
		# how thick my plating is
		self.plating = 3

		self.actiontime = 0
		self.actiontext = ""
		
		self.Rescale()
		
		self.viewingInventory = False
	
	def Update(self):
		self.Pump()
		if self.invisible:
			self.invisible -= 1
		if self.invulnerable:
			self.invulnerable -= 1
		if self.confused:
			self.confused -= 1
		if self.actiontime:
			self.actiontime -= 1
	
	def SetActionText(self, txt):
		self.actiontime = 60
		self.actiontext = txt
	
	#### Actions
	
	def Invulnerable(self, amount):
		self.invulnerable = amount
	
	def Confused(self, amount):
		self.confused = amount
	
	def Invisible(self, amount):
		self.invisible = amount
	
	def Plating(self, amount):
		self.plating += amount
	
	def Strength(self, amount):
		self.strength += amount
		self.TestDeath()
	
	def Teleport(self):
		print "Teleport"
	
	def KnowAll(self):
		for i in self.inventory:
			self.known[(i.istext, i.amount)] = True
	
	def Poem(self):
		self.actiontime = 120
	
	def TestDeath(self):
		if self.strength < 0:
			self.viewingInventory = False
			self.Animate("dead")
	
	#### KEYS
	
	def KeysHeld(self, e):
		if not self.viewingInventory and self.strength >= 0:
			self.lastposition = self.position[:]

			if e.IsHeld(K_UP):
				self.Move(1, -self.velocity)
				if not (e.IsHeld(K_LEFT) or e.IsHeld(K_RIGHT)):
					self.Animate("walkup")
			elif e.IsHeld(K_DOWN):
				self.Move(1, self.velocity)
				if not (e.IsHeld(K_LEFT) or e.IsHeld(K_RIGHT)):
					self.Animate("walkdown")
			
			if e.IsHeld(K_RIGHT):
				self.Move(0, self.velocity)
				self.Animate("walkright")
			elif e.IsHeld(K_LEFT):
				self.Move(0, -self.velocity)
				self.Animate("walkleft")
		elif self.strength < 0:
			self.Animate("dead")
		else:
			self.Animate("stand")
	
	def KeyDown_tab(self, e):
		if self.strength >= 0:
			self.viewingInventory = not self.viewingInventory
	
	def KeyUp(self, e):
		if self.strength >= 0:
			self.Animate("stand")
	
	def KeyDown_space(self, e):
		if self.viewingInventory and self.selected != None:
			self.known[(self.inventory[self.selected].istext, self.inventory[self.selected].amount)] = True
			self.SetActionText(self.inventory[self.selected].usetext)
			self.inventory[self.selected].Action(self, self.inventory[self.selected].amount)
			self.inventory.remove(self.inventory[self.selected])
			if len(self.inventory):
				self.selected = 0
			else:
				self.selected = None
			self.viewingInventory = False
		else:
			if len(self.inventory) < 8:
				if self.selected == None:
					self.selected = 0
				self.inventory.extend(self.nearstuff)
				self.world.PickUp(self.nearstuff)
	
	def KeyDown_delete(self, e):
		if self.viewingInventory and self.selected != None:
			self.inventory.remove(self.inventory[self.selected])
			if len(self.inventory):
				self.selected = 0
			else:
				self.selected = None
	
	def KeyDown_down(self, e):
		if self.viewingInventory and self.selected != None:
			self.selected = min(self.selected + 1, len(self.inventory) - 1)
	
	def KeyDown_up(self, e):
		if self.viewingInventory and self.selected != None:
			self.selected = max(self.selected - 1, 0)
	
	def Collide(self, entity):
		if self.strength >= 0:
			self.position = self.lastposition
			self.Animate("stand")
	
	def NearItem(self, e):
		"""
		An item is close to us.
		"""
		self.nearstuff.append(e)
	
	def ForgetNearStuff(self):
		"""
		Forget all the stuff that is near to us every frame.
		"""
		self.nearstuff = []
	
