from math import *
from engine.Gfx import gfx
from engine.EventHandler import events, EventMonitor
from engine.Entity import Entity
from engine import Randomness
from Scenery import Scenery

class Stone(EventMonitor, Entity, Scenery):
	def __init__(self, world, x, y):
		EventMonitor.__init__(self)
		Entity.__init__(self, world)
		
		self.position = [x, y]
		
		self.scale = 20
		
		self.GenerateRock()
		self.AddAnimation("rock", self.data, [self.Normalise])
		self.Animate("rock")
		
		self.Rescale()
	
	def Update(self):
		self.Rescale()
	
	def GenerateRock(self):
		"""
		Generates a stone using random polygons, dependent on our location.
		Hooray!
		"""
		self.data = []
		angle = 0

		# add some random variation to the position of this guy
		self.position[0] += Randomness.Noise2D(self.position[0], self.position[1], "rockposx") * 0.25
		self.position[1] += Randomness.Noise2D(self.position[0], self.position[1], "rockposy") * 0.25

		self.scale = Randomness.ScaledNoise2D(self.position[0] * 10, self.position[1] * 10, "rockscale") * 50
		
		sides = int(Randomness.ScaledNoise2D(self.position[0] * 10, self.position[1] * 10, "rocksides") * 5) + 3
		c = min(1.99999, (max(0.3, Randomness.ScaledNoise2D(self.position[0], self.position[1], "rockcolor")) - 0.3) * 4.2)
		colors = [[200, 150, 0], [100, 100, 100], [20, 150, 250]]
		color = [Randomness.CosInterpolation(colors[int(c)][x], colors[int(c) + 1][x], c - int(c)) for x in range(3)]
		
		points = []
		pos = [0, 0]
		
		lengths = [1, 3, 3, 3, 4, 4]
		
		for i in range(sides):
			points.append(pos[:])
			pos[0] += cos(angle)
			pos[1] += sin(angle)
			angle -= (pi * 2) / sides
			if i > 0:
				pos[0] += Randomness.ScaledNoise2D(self.position[0] * 100.0, self.position[1] * 100.0, ("stonex", i)) - 0.5
				pos[1] += Randomness.ScaledNoise2D(self.position[0] * 100.0, self.position[1] * 100.0, ("stoney", i)) - 0.5

		self.data = [[{'outline': [0, 0, 0], 'color': color, 'width': 1, 'points': points, 'closed': True, 'filled': True}]]
		
		shadow = []
		for p in points[0: lengths[sides - 3]]:
			shadow.append([p[0] - 0.05, p[1] - 0.05])
		
		shadow[0][0] += 0.1
		shadow[-1][1] += 0.1
		
		bshadow = []
		for p in shadow[1:-1]:
			bshadow.append([p[0], p[1]])
		
		bshadow.reverse()
		
		for p in bshadow:
			p[0] -= 0.2
			p[1] -= 0.2
		
		shadow.extend(bshadow)
		# add the shadow
		# this is a polygon which starts at the -3rd point
		
		self.data[0].insert(0, {'outline': [0, 0, 0], 'color': self.Darken(color, 20), 'width': 0, 'points': shadow, 'closed': True, 'filled': True})

