from engine.Gfx import gfx
from engine.EventHandler import events, EventMonitor
from engine.Entity import Entity
from engine import Randomness
from math import *
from Scenery import Scenery

def treeAt(x, y):
	return Randomness.InterpolatedNoise2D(x * 100, y * 100, "treesize") > 0.5

class Tree(EventMonitor, Entity, Scenery):
	def __init__(self, world, x, y):
		EventMonitor.__init__(self)
		Entity.__init__(self, world)
		
		self.scale = 0.3 #Randomness.random() / 3.0
		self.position = [x, y]

		self.GenerateTree()
		self.AddAnimation("tree", self.data, [self.Resize])
		self.Animate("tree")
		
		self.Rescale()
	
	def Update(self):
		self.Rescale()
	
	def GenerateTree(self):
		"""
		Generates a tree form from a Lindenmayer system calculated randomly, dependent on our location.
		Hooray!
		
		F = draw forward
		f = move forward with no replacement rule
		R = turn right
		L = turn left
		P = push stack
		p = pop stack
		
		T = thicker
		t = thinner
		
		G = greener
		g = less green
		B = browner
		b = less brown
		P = purpler
		p = less purple
		"""
		alphabet = "FfRLTtPp"
		self.position[0] += Randomness.Noise2D(self.position[0], self.position[1], "treex") * 0.25
		self.position[1] += Randomness.Noise2D(self.position[0], self.position[1], "treey") * 0.25
		
		# length of one step
		# step = Randomness.random()
		step = 1 # Randomness.InterpolatedNoise2D(self.position[0] * 1000, self.position[1] * 100, "treesize")
		self.scale = Randomness.ScaledNoise2D(self.position[0] * 10, self.position[1] * 10, "treesize") * 0.5
		#print self.scale
		# angle of a turn
		# turn = Randomness.random() * pi/4.0
		turn = Randomness.InterpolatedNoise2D(self.position[0] * 10, self.position[1] * 10, "treeturn")
		
		# number of times to iterate to generate the tree
		# iterations = Randomness.randint(1, 5)
		iterations = int(Randomness.InterpolatedNoise2D(self.position[0] * 10, self.position[1] * 10, "treeiterations") * 2 + 1)

		# colors for plants
		colors = [[10, 200, 10], [150, 10, 200], [160, 150, 10], [0, 180, 180], [10, 200, 10], [10, 200, 10]]
		colorstart = colors[int(Randomness.InterpolatedNoise2D(self.position[0], self.position[1], "treecolorstart") * 5.9999)]
		colorstop = colors[int(Randomness.InterpolatedNoise2D(self.position[0], self.position[1], "treecolorstop") * 5.9999)]
		
		# initial string
		# string = "".join([Randomness.choice(alphabet) for i in range(Randomness.randint(1,10))])
		strings = ["fftfftPRFpPLFpfftF", "fftLfRfFtRfLfFPRfffpPLfffpF", "PRffFptPLfFfpFttFT", "fffPfftPLffFpttPRffFpFp", "fFftPfftRFftRfftpfPfftLFftLfftpFt", "PRFtRftpfPLFtLftptfF"]
		# string = strings[5]
		string = strings[int(Randomness.ScaledNoise2D(self.position[0] * 10, self.position[1] * 10, "treetype") * 5.999999)]
		# string = "FFFF"
		# initial ruleset
		# rule = dict([(l, "".join([Randomness.choice(alphabet) for i in range(Randomness.randint(1,3))])) for l in alphabet])
		rule = {"F": string}
		#rule = {"F": "F"}
		#rule = {"F": "".join([Randomness.choice(alphabet) for i in range(Randomness.randint(2,10))])}
		# rule = {"A": "BAB", "B": "AAABA"}
		# stack to remember position, thickness, color, angle
		stack = []
		
		# position
		pos = [0, 0]
		# color
		color = colorstart
		# angle = 0
		angle = 0
		# thickness of the piece
		thickness = 6
		# result
		self.data = [[]]
		
		# calculate the tree
		for i in range(iterations):
			string = "".join([((rule.has_key(c) and rule[c]) or c) for c in string])
		
		# now turn the tree into lines
		for c in string:
			# push onto the stack
			if (c == 'P'):
				stack.append([pos, color, thickness, angle])
			if (c == 'p'):
				if len(stack):
					[pos, color, thickness, angle] = stack.pop()
			if (c == 'F' or c == 'f'):
				new = [pos[0] + sin(angle) * step, pos[1] + -cos(angle) * step]
				self.data[0].append({'outline': color, 'color': [0, 0, 0], 'width': thickness, 'points': [pos, new], 'closed': False, 'filled': False})
				pos = new
			if (c == 'R'):
				angle += turn
			if (c == 'L'):
				angle -= turn
			if (c == 'T' and thickness < 5):
				thickness += 1
				color = self.ColorCloser(color, colorstop)
			if (c == 't' and thickness > 1):
				thickness -= 1
				color = self.ColorCloser(color, colorstop)
		
		# if we didn't create anything at all, make a tiny little stalk
		if not len(self.data[0]):
			self.data = [[{'outline': color, 'color': [0, 0, 0], 'width': thickness, 'points': [[0, 0], [0, 0.1]], 'closed': False, 'filled': False}]]

	
	def ColorCloser(self, orig, dest):
		"""
		Change colors, but never reach your destination.
		Ahhh, Xeno.
		"""
		nu = [0, 0, 0]
		for p in range(3):
			nu[p] = Randomness.CosInterpolation(orig[p], dest[p], 0.3)
		return nu

